//設定用
const Config = {
  //画面の解像度
  Screen: {
    Width: 256,//幅
    Height: 256,//高さ
    BackGroundColor: 0x444444,//背景色
  },
  Keys: { //キーボード入力
    Up: "w",
    Right: "d",
    Down: "s",
    Left: "a",
    A: "n",
    B: "m",
    Start: "Enter"
  },
}

//読みこむデータリスト(名前：ファイルパス)
const Resource = {
  Player: 'assets/player.png',
}

//上のリストを読み込みやすい形に変えてます
const assets = [];
for (let key in Resource) {
  assets.push(Resource[key]);
}

let core;//ゲームの基幹プログラム用の変数
let inputManager;//入力管理用

//ブラウザの読み込みが完了したら実行されます
window.onload = () => {
  //設定した画面サイズでcoreを作成
  core = new Game(Config.Screen.Width, Config.Screen.Height, Config.Screen.BackGroundColor);
  inputManager = new InputManager();

  //データのロード
  core.preload(assets);

  //読み込み完了でメインシーンに切り替わります
  core.onload = () => {
    core.replaceScene(new MainScene());
  }
}

let player;
let bullet;
//メインシーン
class MainScene extends Container {
  constructor(){
    super();

    //プレイヤー
    player = new EnchantSprite(16, 16);
    this.addChild(player);
    player.image = core.resources[Resource.Player].texture; 
    player.position.set(Config.Screen.Width*0.5, Config.Screen.Height*0.5);
    player.anchor.set(0.5, 0.5)

    //弾
    bullet = new Graphics();
    this.addChild(bullet);
    bullet.circFill(0, 0, 4, 0xff0000);
    bullet.y = -10;
  }
  update(delta){
    super.update(delta);
    const oblique = 1 / Math.sqrt(2);//ななめ移動の値
    switch(inputManager.checkDirection()){
      case inputManager.keyDirections.UP:
        player.y--;
        break;
      case inputManager.keyDirections.UP_RIGHT:
        player.x += oblique;
        player.y -= oblique;
        break;
      case inputManager.keyDirections.RIGHT:
        player.x++;
        break;
      case inputManager.keyDirections.DOWN_RIGHT:
        player.x += oblique;
        player.y += oblique;
        break;
      case inputManager.keyDirections.DOWN:
        player.y++;
        break;
      case inputManager.keyDirections.DOWN_LEFT:
        player.x -= oblique;
        player.y += oblique;
        break;
      case inputManager.keyDirections.LEFT:
        player.x--;
        break;
      case inputManager.keyDirections.UP_LEFT:
        player.x -= oblique;
        player.y -= oblique;
        break;
      default:
        break;
    }
    
    //Aボタンが押されたとき
    if(inputManager.checkButton("A") == inputManager.keyStatus.DOWN){
      if(bullet.y <= -20){
        bullet.clear();
        bullet.circFill(0, 0, 4, 0xff0000);
        bullet.y = player.y;
        bullet.x = player.x;
      }
    }
    //Bボタンが押されたとき
    if(inputManager.checkButton("B") == inputManager.keyStatus.DOWN){
      if(bullet.y <= -20){
        bullet.clear();
        bullet.circFill(0, 0, 8, 0x00ffff);
        bullet.y = player.y;
        bullet.x = player.x;
      }
    }
    //スタートボタンが押されたとき
    if(inputManager.checkButton("Start") == inputManager.keyStatus.DOWN){
      core.pushScene(new PauseScene());
    }   

    //弾の移動
    bullet.y -= 4;
  }
}

//ポーズシーン
class PauseScene extends Container {
  constructor(){
    super();
    const t = new PIXI.Text('PAUSE', new PIXI.TextStyle({
      fontFamily: 'sans-serif',
      fontSize: 32,
      fill: 0xffffff,
    }));
    t.anchor.set(0.5, 0.5);
    t.position.set(Config.Screen.Width*0.5, Config.Screen.Height*0.5);
    this.addChild(t);
  }
  update(delta){
    super.update(delta);
    if(inputManager.checkButton("Start") == inputManager.keyStatus.DOWN){
      core.popScene();
    }  
  }
}

//インプットマネージャー
class InputManager {
  constructor() {
    //方向入力チェック用定数
    this.keyDirections = {
      UP: 1,
      UP_RIGHT: 3,
      RIGHT: 2,
      DOWN_RIGHT: 6,
      DOWN: 4,
      DOWN_LEFT: 12,
      LEFT: 8,
      UP_LEFT: 9,
    };
    //キーの状態管理定数
    this.keyStatus = {
      HOLD: 2,
      DOWN: 1,
      UNDOWN: 0,
      RELEASE: -1,
    };
    //キーの状態管理用変数
    this.input = {
      //入力されたキーのチェック用
      keys: {
        Up: false,
        Right: false,
        Down: false,
        Left: false,
        A: false,
        B: false,
        Start: false
      },
      //一つ前のキーの状態管理用
      keysPrev: {
        Up: false,
        Right: false,
        Down: false,
        Left: false,
        A: false,
        B: false,
        Start: false
      },
    };

    //キーを押した時
    document.addEventListener('keydown', (e) => {
      switch(e.key){
        case Config.Keys.Up:
          this.input.keys.Up = true;
          break;
        case Config.Keys.Down:
          this.input.keys.Down = true;
          break;
        case Config.Keys.Right:
          this.input.keys.Right = true;
          break;
        case Config.Keys.Left:
          this.input.keys.Left = true;
          break;
        case Config.Keys.A:
          this.input.keys.A = true;
          break;
        case Config.Keys.B:
          this.input.keys.B = true;
          break;
        case Config.Keys.Start:
          this.input.keys.Start = true;
          break;
      }
    });

    //キーを離したとき
    document.addEventListener('keyup', (e) => {
      switch(e.key){
        case Config.Keys.Up:
          this.input.keys.Up = false;
          break;
        case Config.Keys.Down:
          this.input.keys.Down = false;
          break;
        case Config.Keys.Right:
          this.input.keys.Right = false;
          break;
        case Config.Keys.Left:
          this.input.keys.Left = false;
          break;
        case Config.Keys.A:
          this.input.keys.A = false;
          break;
        case Config.Keys.B:
          this.input.keys.B = false;
          break;
        case Config.Keys.Start:
          this.input.keys.Start = false;
          break;
      }
    });
  }

  //方向キー入力チェック
  checkDirection() {
    let direction = 0;//初期化
    if(this.input.keys.Up){
        direction += this.keyDirections.UP;
    }
    if(this.input.keys.Right){
      direction += this.keyDirections.RIGHT;
    }
    if(this.input.keys.Down){
      direction += this.keyDirections.DOWN;
    }
    if(this.input.keys.Left){
      direction += this.keyDirections.LEFT;
    }
    return direction;
  }

  //ボタンの入力状態をチェックして返す
  checkButton(key) {
    if(this.input.keys[key]){
      if(this.input.keysPrev[key] == false){
        this.input.keysPrev[key] = true;
        return this.keyStatus.DOWN;//押されたとき
      }
      return this.keyStatus.HOLD;//押しっぱなし
    }else{
      if(this.input.keysPrev[key] == true){
        this.input.keysPrev[key] = false;
        return this.keyStatus.RELEASE;//ボタンを離した時
      }
      return this.keyStatus.UNDOWN;//押されていない
    }
  }
}
